/****************************************************************
 * main file for the game
 * by Wil
 * License CC BY 4.0
 ****************************************************************/

#include <stdio.h> 
#include <stdlib.h>

#include "presets.h"
#include "main.h"
#include "game.h"
#include "menu.h"
#include "musicandsound.h"
#include "../utilitylib/utilitylib.h"
#include "../levels/levelviewer.h"
#include "../levels/levels.h"
#include "../sprites/sprites.h"
#include "dialog_and_item_functions.h"

#include <frodosC64lib.h>
#include "displaytitle.h"
#include "splitraster_isr.h"

uint8_t current_room;
uint8_t active_scene;
void (*custom_function)(void);

uint8_t my_map_pos;

uint8_t spriteNPClink[7];

uint8_t teleportlink[SCREEN_WIDTH_TILES*SCREEN_HEIGHT_TILES];
teleport_entry_t *teleports_current_room;
walkable_entry_t *walkables_current_room;

uint8_t check_walkable_conditions(walkable_entry_t *wt_entry) {
  uint8_t i,cond;
  
  for (i=0;i<N_POSITIVE_CONDITIONS;i++) {
    cond = wt_entry->positive_conditions[i];
    if (cond==0) continue;
    if (!is_f_set(cond)) return 0; // not fulfilled
  }

  for (i=0;i<N_NEGATIVE_CONDITIONS;i++) {
    cond = wt_entry->negative_conditions[i];
    if (cond==0) continue;
    if (is_f_set(cond)) return 0; // not fulfilled
  }    
  return 1; //all conditions fulfilled
}

void update_walkable_tiles(uint8_t room) {
  uint8_t i;
  walkable_entry_t wt_entry;
  
  walkables_current_room = rooms[room-1].walkables;
  
  for(i=0;i<rooms[room-1].no_walkables;i++) {
    wt_entry=walkables_current_room[i];

    if (check_walkable_conditions(&wt_entry))  {
      //make it walkable    
      collisionmap[wt_entry.yposition*SCREEN_WIDTH_TILES + wt_entry.xposition] = 0;
      if (wt_entry.tile_when_walkable != NO_TILE) draw_tile_macro(wt_entry.tile_when_walkable,wt_entry.xposition,wt_entry.yposition);
    } else {
      //make it unwalkable
      collisionmap[wt_entry.yposition*SCREEN_WIDTH_TILES + wt_entry.xposition] = WALKABLE_TILES;     
      if (wt_entry.tile_when_unwalkable != NO_TILE) draw_tile_macro(wt_entry.tile_when_unwalkable,wt_entry.xposition,wt_entry.yposition);      
    }
  }
}

void init_teleportlinks(uint8_t room) {
  uint8_t i;
  teleport_entry_t tp_entry;
      
  for(i=0;i<SCREEN_WIDTH_TILES*SCREEN_HEIGHT_TILES;i++) {
    teleportlink[i]=NO_ACTION;
  }
  teleports_current_room = rooms[room-1].teleportpoints;
  
  for(i=0;i<rooms[room-1].no_teleports;i++) {
    tp_entry=teleports_current_room[i];
    
    teleportlink[tp_entry.from_yposition*SCREEN_WIDTH_TILES + tp_entry.from_xposition] = i | TELEPORT_FL;
    
#if DEBUGLVL==1    
    gotoxy(tp_entry.from_xposition*2,2+tp_entry.from_yposition*2);
    printf("tp");
#endif    
    
  }
}

void place_NPCs(uint8_t room) {
  uint8_t i,j;
  
  j=0;
  for(i=0;i<MAXNPCs;i++) {
    while(NPCs[j].room_index != room) {
      if (NPCs[j].sprite_index==0) break;
      j++;
    }
    if (NPCs[j].sprite_index==0) break;
    
    spriteNPClink[i]=j;
    setSpriteCostume(i+1,NPCs[j].sprite_index);
    updateSpriteAttributes(i+1);
    setSpriteXY(i+1,NPCs[j].xposition * 16 + SPRITEOFFSETX, NPCs[j].yposition * 16 + SPRITEOFFSETY);
    showSprite(i+1);
    j++;
  }
  for(;i<MAXNPCs;i++) {
     hideSprite(i+1);
     spriteNPClink[i]=0xFF;
  }
}

void paint_black_first_line() {
  int i;
  for(i=0;i<40;i++) {
    poke(SCREEN_BASE+i,0);
    poke(COLRAM_BASE+i,0);    
  }
}


void execute_teleport(teleport_entry_t *tp_entry) {
  uint8_t i,cond;
  
  for (i=0;i<N_POSITIVE_CONDITIONS;i++) {
    cond = tp_entry->positive_conditions[i];
    if (cond==0) continue;
    if (!is_f_set(cond)) return; // no teleport
  }

  for (i=0;i<N_NEGATIVE_CONDITIONS;i++) {
    cond = tp_entry->negative_conditions[i];
    if (cond==0) continue;
    if (is_f_set(cond)) return; // no teleport
  }    

  current_room=tp_entry->to_room_index;
  playerx=tp_entry->to_xposition * 8 + SPRITEOFFSETX/2;
  playery=tp_entry->to_yposition * 16 + SPRITEOFFSETY;
}

/* -------------------------------------
   Game loop
   You might want to edit this for your game   
   ------------------------------------- */

void gameloop()
{
  uint8_t collision;
 
  collision=0;
  
  current_room=STARTING_ROOM;
  active_scene=0;
  
  init_map();
  init_sprites();
  
  while(1) {
    if (current_room != active_scene) {
      poke(PASSX,0);
      poke(PASSY,current_room*12-12);      
      draw_map();
      active_scene=current_room;
      
      place_NPCs(current_room);
      
      custom_function=rooms[current_room-1].custom_function;
      paint_black_first_line();
      
      init_teleportlinks(current_room);
      update_walkable_tiles(current_room);
    }
    
    enable_movement();
    
    my_map_pos=map_idx; //this gets the map index that was set by the interrupt routine
    
#if DEBUGLVL==1    
    gotoxy(0,0);
    printf("idx:%i  ",my_map_pos);
#endif
    collision=get_collision_index();
    
    if ((joystick_fire(2)) && (collision) ) {
      start_dialog(NPCs[spriteNPClink[collision-1]].dialog, NPCs[spriteNPClink[collision-1]].dialogsize);
      update_walkable_tiles(current_room);      
    }
  
    if (custom_function) custom_function();
    
    //check teleport points
    
    if (teleportlink[my_map_pos] & (TELEPORT_FL)) {
    
#if DEBUGLVL==1    
    gotoxy(10,0);
    printf("tp:%i",teleportlink[my_map_pos]);
    
#endif    
    
     execute_teleport(&teleports_current_room[teleportlink[my_map_pos] & 0x3f]);
    }    
    
#if DEBUGLVL==1        
  if (peek(203)==54) //up arrow = skip 1 room forward
    if (current_room < MAX_ROOMS-1) current_room++;
  if (peek(203)==57) //down arrow = skip 1 room forward    
    if (current_room>1) current_room--;

  if (peek(203)==48) //Pound sign = list enabled flags
    {
    int i;
    gotoxy(0,0);
    textcolor(1);
    for(i=FLAG_EXIT; i<MAX_FLAGS; i++)
      if (is_f_set(i)) cprintf("%d,",i);
    }
#endif    
    
        
  }
}

void endscreen()
{

    playerx=150;
    delayms(1000);
    disable_movement();
        
    gotoxy(0,12);
    
    printf("game over!\n\n");
    delayms(3000);
}
