; splitraster
; 
; Jun-2020 V0.4
; Wilfried Elmenreich
; Code under CC-BY-4.0 license

.include "presets.inc"
.include "LAMAlib.inc"

.export _initcolram := initcolram
.export _init_splitraster_menu := init_splitraster_menu
;.export _clrhalfscreen

music_IRQ_routine=MUSIC_BASE+3

dbg_borders=0
enable_blink=0

first_line=0
second_line=50+MENUTITLEPICSCREEN_HEIGHT*8
third_line=248

;-----------------------------------------------
;call init_split raster tostart rasterplit
;-----------------------------------------------
init_splitraster_menu:
	sei

	;set irq sources
	lda #$7f
	and $D011	; accu is still $7f
	sta $D011 	; Clear most significant bit in VIC's raster register

	sta $dc0d	; disable timer interrupts
	sta $dd0d
	lda $dc0d	; acknowledge CIA interrupts

	lda #<first_line		; Set the raster line number where interrupt should occur 
	sta $D012 	
	lda $D011
	.if >first_line = 0
	  and #$7f
	.else
	  ora #$80
	.endif
	sta $D011

	lda #01		; set raster interrupt
	sta $D01A 	

	;init colram upper half for logo

	ldy #0


	;set ISR addr
	lda #<isr0
	sta $314
	lda #>isr0
	sta $315

;music init is done in main.c
;.ifdef music_IRQ_routine
;        lda #$00
;	jsr MUSIC_BASE
;.endif
	lda 1
	pha
	
	poke 1,$35	;RAM visible at $A000-$BFFF and $E000-$FFFF

	ldy #00
initcolram:
	lda MENUTITLEPIC_COLRAM_BASE,y
	sta $d800,y
	iny
	cpy #MENUTITLEPICSCREEN_HEIGHT*40
	bne initcolram

	lda #00
	sta $ffff    ;to avoid VIC $3fff patterns in border

	pla
	sta 1

	cli

	rts

_clrhalfscreen:
	ldy #39
	lda #$20
clrloop:
	lda #$20
	.repeat 25-MENUTITLEPICSCREEN_HEIGHT,I
	sta SCREEN_BASE+(I+MENUTITLEPICSCREEN_HEIGHT)*40,y
	.endrep
	dey
	bpl clrloop
	rts

;-----------------------------------------------
;IRQ routine for line 0
;-----------------------------------------------
isr0:	
.if dbg_borders=1
	lda #1
	sta $D020	;white, for debug only
.endif
	;poke(53265,27) ;normal borders


.if enable_blink=1
	;make pic blink
.endif


	;enable bitmap graphics
	lda #59   ;bit 5 on = Bitmap-Modus, normal borders
	sta $D011 
	lda #216
	sta $D016
	lda #%10101000
	sta $d018	;screen at VIC bank + $2800, bitmap at +$2000

	;set next ISR
	lda #<isr1
	sta $314
	lda #>isr1
	sta $315
	asl $D019 ; clear VIC's interrupt flag

nextrasterline:
	lda #<second_line
	sta $D012
	lda $D011
	.if >second_line = 0
	  and #$7f
	.else
	  ora #$80
	.endif

	ldy #0
	sty $d020	;gray bg for logo

	sta $D011

	sty $d021

.ifdef music_IRQ_routine
	jsr music_IRQ_routine
.endif
	jmp $ea31

blinkstate:
.byte   00

;-----------------------------------------------
;IRQ routine for line inbetween
;-----------------------------------------------
isr1:	
	asl $D019 ; clear VIC's interrupt flag

	lda #27  ;bitmap off
d018_lonibble .set (CHARSET_BASE-VIC_BANK)/$400
d018_hinibble .set (SCREEN_BASE-VIC_BANK)/$40
        ldx #d018_hinibble+d018_lonibble
	;ldx #%00110100	 ;screen at VIC bank + $0c00, charset at $d000
	ldy #200 ;multicolor off
	sta $D011
	stx $D018

	lda #0
	sta $d020
	sta $d021

	sty $D016

.if dbg_borders=1
	lda #5
	sta $D020	;green, for debug only
.else
	nop
.endif


	lda #<isr2
	sta $314
	lda #>isr2
	sta $315

	lda #<third_line
	sta $D012
	lda $D011
	.if >third_line = 0
	  and #$7f
	.else
	  ora #$80
	.endif
	sta $D011

	jmp $ea81

;-----------------------------------------------
;IRQ routine for opening border
;-----------------------------------------------
isr2:	
	lda #19  ;24 row border
	sta $D011	
	
.if dbg_borders=1
	lda #4
	sta $D020	;purple, for debug only
.endif

	lda #<isr0
	sta $314
	lda #>isr0
	sta $315
	asl $D019 ; clear VIC's interrupt flag

	lda #<first_line		
	sta $D012 	
	lda $D011
	.if >first_line = 0
	  and #$7f
	.else
	  ora #$80
	.endif
	sta $D011

	jmp $ea81
