;------------------------------------------------------
; Dec-2020 V0.3
; Wilfried Elmenreich
; Code under CC-BY-4.0 license
;------------------------------------------------------
; Displays title image and waits for key/joyclick
; we assume the following memory layout:
; somewhere before $c800: this code
; $c800 color ram data
; $cc00 screen ram (colors 1 & 2)
; $e000 bitmap data
;------------------------------------------------------

.include "..\src\presets.inc"
.include "LAMAlib.inc"

colramdata=$c800
screen=$cc00

.export _display_titlescreen := display_titlescreen

;copy colorram from C800 to D800

display_titlescreen:

.if ENABLE_TITLEPICMUSIC>0
	sei
	poke 1,$36
	lda #00	;select tune 0
        jsr TITLEPIC_MUSIC_BASE	; init music
        ; now set the new interrupt pointer
	pokew $314,_interrupt ; point IRQ Vector to our custom irq routine
	set_raster_irq 250	;change IRQ source to VIC raster interrupt, make IRQ at rasterline 250
        cli ; clear interrupt disable flag^
.endif

	ldx #TITLEPICBORDERCOLOR
	stx $d020
	ldx #TITLEPICBACKGROUNDCOLOR
	stx $d021

	.if TITLEPICBACKGROUNDCOLOR > 0
	ldx #00
	.endif
loop:
.REPEAT 4,I
	lda colramdata+I*$100,x
	sta COLRAM_BASE+I*$100,x
.ENDREP
	dex
	bne loop

;enable graphics mode configure VIC bank
	lda #59   ;bit 5 on = Bitmap-Modus
	sta $D011 
	lda #216
	sta $D016

	lda #%00111000
	sta $d018	;screen at VIC bank + $0c00, bitmap at +$2000

	lda $dd00
	and #%11111100
	sta $dd00	;set VIC bank to $C000-$FFFF

; waits for any key to be pressed and call sprite animation

buttonchk:	
.if EXITJOYCLICK=1
	lda $dc02-JOYPORT
	and #$10
	beq exitcheck
.endif

.if EXITKEYPRESS=1
	lda #00
	sta $dc00
	lda $dc01
        cmp #$ff
        bne exitcheck
	lda #$7f
	sta $dc00
.endif
	jmp buttonchk 
exitcheck:

;------------------------------------------------------
; Paint the pic black to hide any ongoing calculations
;------------------------------------------------------

	;bg/fg color in y
	ldx #00
	txa
loop2:	
	sta screen,x
	sta screen+$100,x
	sta screen+$200,x
	sta screen+$300,x	;this overwrites sprite pointers, sry
	sta COLRAM_BASE,x
	sta COLRAM_BASE+$100,x
	sta COLRAM_BASE+$200,x
	sta COLRAM_BASE+$300,x	
	inx
	bne loop2

.if ENABLE_TITLEPICMUSIC>0

dummy_isr_addr=$32e
;dummy_isr_addr=$3f0

	sei
	;install a dummy isr
	for X,5,downto,0
	  lda dummy_isr_prg,x
	  sta dummy_isr_addr,x
	next

	pokew $314,dummy_isr_addr
	poke $d418,0
	poke 1,$37
	poke 53265,0	;screen off
	cli
.endif
	jmp PROGRAMSTARTADDRESS

;------------------------------------------------------
; Interrupt service routine calling the music player
;------------------------------------------------------

.if ENABLE_TITLEPICMUSIC>0
.proc _interrupt
	asl $d019 ; acknowledge raster interrupt
	;inc $d020
        jsr TITLEPIC_MUSIC_BASE+3
	;dec $d020 ; flash border to see we are live
        jmp $EA31 ; do the normal interrupt service routine
.endproc

dummy_isr_prg:
	asl $d019
	jmp $ea31
dummy_isr_end:
.endif